import logging
import sys

import click
import click_log

from . import repo_has_same_ast


def _get_logger():
    logger = logging.getLogger(__name__)
    click_log.basic_config(logger)
    return logger


@click.command()
@click.option('--git-dir', default='.', type=click.Path(exists=True, file_okay=False))
@click_log.simple_verbosity_option(_get_logger())
def main(git_dir):
    try:
        has_same_ast = repo_has_same_ast(git_dir, _get_logger())
    except ValueError as e:
        raise click.BadParameter(e)
    if has_same_ast:
        sys.exit(0)
    else:
        sys.exit(1)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages


with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name='python-compare-ast',
    version='1.0.0',
    description='Compares ASTs of Python files that have been changed in git',
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/omegacen/python-compare-ast",
    license='LGPL-3.0-or-later',
    author='Teake Nutma',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'python-compare-ast = python_compare_ast.__main__:main'
        ]
    },
    python_requires='>=3.6',
    install_requires=[
        'gitpython',
        'click',
        'click-log'
    ]
)



import ast
import os

import git


def repo_has_same_ast(git_dir: str, logger) -> bool:
    try:
        repo = git.Repo(git_dir, search_parent_directories=True)
    except git.InvalidGitRepositoryError:
        raise ValueError(f'Not a git dir nor any of its parents: {os.path.abspath(git_dir)}')
    logger.info(f'Analysing {repo.git_dir}')
    if repo.is_dirty():
        logger.info('Repo is dirty, comparing uncommitted changes.')
        file_strategy = dirty_files_strategy
    else:
        logger.info(f'Repo is clean, comparing changes in latest commit ({repo.head.object.hexsha}).')
        file_strategy = lastcommit_files_strategy
    num_files_diff = num_files_different(repo, file_strategy, logger)
    logger.info(f'Found {num_files_diff} AST differences.')
    if num_files_diff:
        return False
    else:
        return True


def num_files_different(repo: git.Repo, file_strategy, logger) -> int:
    num_ast_diffs = 0
    for path, source_original, source_modified in file_strategy(repo):
        if not have_equal_ast(source_original, source_modified):
            logger.warn(f'AST does not match for {path}')
            num_ast_diffs += 1
        else:
            logger.info(f'AST matches for {path}')
    return num_ast_diffs


def lastcommit_files_strategy(repo: git.Repo):
    for item in repo.index.diff('HEAD~1').iter_change_type('M'):
        path = item.a_path
        if path.endswith('.py'):
            yield path, repo.git.show(f'HEAD~1:{path}'), repo.git.show(f'HEAD:{path}')


def dirty_files_strategy(repo: git.Repo):
    for item in repo.index.diff(None):
        path = item.a_path
        if not path.endswith('.py'):
            continue
        source_original = repo.git.show(f'HEAD:{path}')
        with open(os.path.join(repo.working_dir, path)) as modified:
            source_modified = modified.read()
        yield path, source_original, source_modified


def have_equal_ast(source1: str, source2: str) -> bool:
    try:
        dump1 = ast.dump(ast.parse(source1))
        dump2 = ast.dump(ast.parse(source2))
    except SyntaxError:
        return False
    return dump1 == dump2
